@@ -29,6 +29,7 @@ typedef unsigned short int us_int;
 extern volatile sig_atomic_t	quit;
 extern int			debug;
 extern int			mimes_size;
+extern int			fdbsdauth[2];
 extern char			*mimes;
 extern char			logn[1024];
 
@@ -107,6 +108,7 @@ struct cfg {
 	char	htaccess[1024];		/* Filename for basic auth */
 	char	htpasswd[1024];		/* Full Filename for basic auth users */
 	char	file[1024];		/* Full Filename for config file */
+	int	bsdauth;		/* BSD auth Flag */
 	int	ssl;			/* SSL Flag */
 	int	sslport;		/* SSL Listener Port */
 	char	sslportc[8];		/* SSL Listener Port in char */
@@ -115,6 +117,7 @@ struct cfg {
 	char	c401[1024];		/* Filename for 401 custom response */
 	char	c403[1024];		/* Filename for 403 custom response */
 	char	c404[1024];		/* Filename for 404 custom response */
+	char	homedirs[1024];		/* Full Path to homedirs */
 };
 extern struct cfg config;
 
@@ -151,8 +154,8 @@ struct header {
 	char	rp_fv_loc[1024];	/* Response Value Location: */
 	char	rp_fv_auw[1024];	/* Response Value WWW-Authenticate: */
 	char	rp_header[8192];	/* Response Header */
-	int	rp_fsize;		/* Repsonse File Size */
-	int	rp_foffs;		/* Response File Offset */
+	off_t	rp_fsize;		/* Repsonse File Size */
+	off_t	rp_foffs;		/* Response File Offset */
 	int	rp_hsize;		/* Response Header Size */
 	us_int	rp_status;		/* Response Status */
 	us_int	x_chk;			/* Flag for Chunking */
@@ -183,13 +186,13 @@ struct connection {
 	char	ip[24];			/* Connection, Client IP */
 	char	*rbuf;			/* Connection, Read buf */
 	char	*wbuf;			/* Connection, Write buf */
-	int	roff;			/* Connection, Read buf offset */
-	int	wred;			/* Connection, Write buf read bytes */
-	int	wsnd;			/* Connection, Write buf sent bytes */
+	off_t	roff;			/* Connection, Read buf offset */
+	off_t	wred;			/* Connection, Write buf read bytes */
+	off_t	wsnd;			/* Connection, Write buf sent bytes */
 	int	to;			/* Connection, Timeout */
 	int	pfdo;			/* Connection, Open File descriptors */
 	int	pfdn[HDN];		/* Partitial, File descriptor */
-	int	pfds[HDN];		/* Partitial, Bytes sent */
+	off_t	pfds[HDN];		/* Partitial, Bytes sent */
 	int	psta[HDN];		/* Partitial, Response Status */
 	char	*pfdh[HDN];		/* Partitial, Response Header */
 	char	*plreq[HDN];		/* Partitial, Log Request */
