@@ -80,8 +80,12 @@ static const struct {
 	{ "7c", '\x7c' }, { "7d", '\x7d' }, { "7e", '\x7e' }, { "7f", '\x7f' }
 };
 
-static const char *icd = "<img src=/icons/dir.gif>";
-static const char *icf = "<img src=/icons/file.gif>";
+static const char *icd = "<img src=\"/icons/dir.gif\" alt=\"icon\">";
+static const char *icf = "<img src=\"/icons/file.gif\" alt=\"icon\">";
+static const char *met = "<meta http-equiv=\"content-type\" "
+    "content=\"text/html; charset=iso-8859-1\">";
+static const char *doc =
+    "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">";
 
 /*
  * http_verify()
@@ -143,9 +147,9 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 {
 	DIR		*odir;
 	int		cpid, file, fds1[2], fds2[2];
-	int		i, r, s, rp, rt, sp, st, len, size;
+	int		i, r, s, rp, rt, sp, st, len, size, dirents;
 	us_int		x_nph, x_fork, x_cpage;
-	char		*b, *h, *x = NULL;
+	char		**dirsort, *b, *h, *x = NULL;
 	char		ch[1], image[128], tmp[1024], full[1024], status[1024];
 	char		buf[BS], cpage[1024], *cgiarg[2], *cgienv[64];
 	struct tm	*t = NULL;
@@ -634,30 +638,52 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 		else
 			sys_write(sfd, rh->rp_header, strlen(rh->rp_header));
 
-		/* open directory */
-		if ((odir = opendir(rh->rq_filep)) == NULL)
-			exit(1);
-
 		/* initialize */
 		memset(buf, 0, sizeof(buf));
 
 		/* create html title */
-		snprintf(buf, sizeof(buf), "<html>\n<head><title>Index of %s"
-		    "</title></head>\n<body>\n\n<h1>Index of %s</h1>\n<hr>\n"
+		snprintf(buf, sizeof(buf),
+		    "%s\n<html>\n<head>\n<title>Index of %s</title>\n"
+		    "%s\n</head>\n<body>\n\n<h1>Index of %s</h1>\n<hr>\n"
 		    "<table cellpadding=2 cellspacing=5>\n"
 		    "<tr><td><b>Type</b></td><td><b>Filename</b></td>"
 		    "<td><b>Last Modified</b></td><td><b>Size</b></td></tr>\n",
-		    rh->rq_index, rh->rq_index);
+		    doc, rh->rq_index, met, rh->rq_index);
 
-		/* list directory content */
-		while ((rdir = readdir(odir)) != NULL) {
+		/* open directory and count file entries */
+		if ((odir = opendir(rh->rq_filep)) == NULL)
+			exit(1);
+		for (dirents = 0; (rdir = readdir(odir)) != NULL;) {
+			/* we dont count hidden files */
+			if (rdir->d_name[0] == '.')
+				continue;
+			dirents++;
+		}
+		closedir(odir);
+		dirsort = malloc((dirents +1) * sizeof(*dirsort));
+		if ((odir = opendir(rh->rq_filep)) == NULL)
+			exit(1);
+
+		/* get directory content */
+		for (i = 0; (rdir = readdir(odir)) != NULL && i < dirents;) {
 			/* we dont list hidden files */
 			if (rdir->d_name[0] == '.')
 				continue;
+			if ((dirsort[i] = strdup(rdir->d_name)) == NULL)
+				exit(1);
+			i++;
+		}
+		closedir(odir);
 
+		/* sort directory content */
+		qsort(dirsort, i, sizeof(dirsort[0]), sys_compar);
+		dirsort[i] = NULL;
+
+		/* print directory content */
+		for (i = 0; dirsort[i] != NULL; i++) {
 			/* create full file path */
 			snprintf(full, sizeof(full), "%s%s", rh->rq_filep,
-			    rdir->d_name);
+			    dirsort[i]);
 
 			/* get file status */
 			stat(full, &sta);
@@ -672,9 +698,9 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 			/* create html file entry */
 			snprintf(tmp, sizeof(tmp),
 			    "<tr><td>%s</td><td><a href=\"%s\">%s</a>"
-			    "</td><td>%s</td><td>%d</td></tr>\n",
-			    image, rdir->d_name, rdir->d_name, http_date(t),
-			    (int)sta.st_size);
+			    "</td><td>%s</td><td>%lld</td></tr>\n",
+			    image, dirsort[i], dirsort[i], http_date(t),
+			    sta.st_size);
 
 			/* buffer full? send it! */
 			if (strlen(tmp) > sizeof(buf) - strlen(buf)) {
@@ -699,10 +725,10 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 
 			/* fill buffer */
 			strlcat(buf, tmp, sizeof(buf));
-		}
 
-		/* close directory */
-		closedir(odir);
+			/* free directory entry */
+			free(dirsort[i]);
+		}
 
 		if (sp != -1) {
 			/* flush buffer */
@@ -723,8 +749,8 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 				free(x);
 
 			/* create html footer */
-			snprintf(buf, sizeof(buf), "</table>\n<hr>\n%s\n"
-			    "</body>\n</html>", http_sig);
+			snprintf(buf, sizeof(buf),
+			    "</table>\n<hr>\n%s\n</body>\n</html>", http_sig);
 			if (rh->x_chk)
 				x = http_chunk(buf, strlen(buf));
 			else
@@ -997,7 +1023,7 @@ http_alog(const int sfd, const int hr)
 	time(&tnow);
 	t = localtime(&tnow);
 
-	snprintf(log_1, sizeof(log_1), "%s - - %s \"%s\" %d %d",
+	snprintf(log_1, sizeof(log_1), "%s - - %s \"%s\" %d %lld",
 	    c[sfd].ip, sys_date(t), c[sfd].plreq[hr], c[sfd].psta[hr],
 	    c[sfd].pfds[hr]);
 	snprintf(log_2, sizeof(log_2), "\"%s\" \"%s\"",
@@ -1200,9 +1226,9 @@ http_body(const char *title, const char *msg, const char *header,
 
 	/* html */
 	snprintf(body, sizeof(body),
-	    "<html>\n<head>\n<title>%s</title>\n</head>\n<body>\n\n"
-	    "<h1>%s</h1>\n%s\n<hr>\n%s\n</body>\n</html>\n",
-	    title, title, msg, http_sig);
+	    "%s\n<html>\n<head>\n<title>%s</title>\n%s\n</head>\n<body>\n\n"
+	    "<h1>%s</h1>\n%s\n<hr>\n%s\n</body>\n</html>",
+	    doc, title, met, title, msg, http_sig);
 
 	/* chunk html */
 	if (chunk)
@@ -1242,7 +1268,8 @@ struct header *
 http_header(const char *header_data, const char *force_status,
     const int force_status_nr, const int blen, const int sfd)
 {
-	int		fsize, file, i, r;
+	off_t		fsize;
+	int		file, i, r;
 	char		line[1024], option[1024];
 	char		alias[1024], cgi_full[1024];
 	char		docroot[1024], file_path[1024];
@@ -1396,7 +1423,7 @@ http_header(const char *header_data, const char *force_status,
 	strcuts(alias, h->rq_uri, '\0', '/', sizeof(alias));
 
 	/* check if uri contains an alias */
-	if (fparse(docroot, alias, config.file, sizeof(tmp)) != -1)
+	if (fparse(docroot, alias, config.file, sizeof(docroot)) != -1)
 		/* yes cut it out */
 		strcuti(file_path, h->rq_uri, strlen(alias),
 		    strlen(h->rq_uri), sizeof(file_path));
@@ -1406,7 +1433,18 @@ http_header(const char *header_data, const char *force_status,
 		strlcpy(file_path, h->rq_uri, sizeof(file_path));
 	}
 
-	/* directory? */
+	/* homedirs */
+	if (config.homedirs[0] != '0') {
+		if (h->rq_uri[1] == '~') {
+			strlcpy(docroot, config.homedirs, sizeof(docroot));
+			strlcpy(file_path, h->rq_uri, sizeof(file_path));
+			/* remove ~ */
+			strlcpy(file_path + 1, h->rq_uri + 2,
+			    sizeof(file_path));
+		}
+	}
+
+	/* directory */
 	if (h->rq_uri[strlen(h->rq_uri) - 1] == '/') {
 		/* save original uri */
 		strlcpy(h->rq_index, file_path, sizeof(h->rq_index));
@@ -1515,7 +1553,7 @@ http_header(const char *header_data, const char *force_status,
 			t = gmtime(&s.st_mtime);
 			strlcpy(h->rp_fv_dam, http_date(t),
 			    sizeof(h->rp_fv_dat));
-			h->rp_fsize = (int)s.st_size;
+			h->rp_fsize = s.st_size;
 		}
 	}
 
@@ -1549,25 +1587,43 @@ http_header(const char *header_data, const char *force_status,
 			    sizeof(h->rq_fv_usr));
 			strcuts(acc_pw, acc_userpw, ':', '\0', sizeof(acc_pw));
 			/* authenticate */
-			if ((r = sys_access_auth(h->rq_fv_usr, acc_pw)) != 1)
-				/* failed */
-				h->x_aut = 0;
+			if (config.bsdauth) {
+				if ((r = sys_access_bsdauth(h->rq_fv_usr,
+				    acc_pw)) != 1)
+					/* failed */
+					h->x_aut = 0;
+			} else {
+				if ((r = sys_access_auth(h->rq_fv_usr,
+				    acc_pw)) != 1)
+					/* failed */
+					h->x_aut = 0;
+			}
 		}
 		/* authenticate option is not set or failed */
 		if (!h->x_aut) {
-			/* get access realm */
-			if (fparse(h->rp_fv_auw, "realm", acc_file,
-			    sizeof(h->rp_fv_auw)) == -1) {
-				/* realm could not be parsed */
-				strlcpy(h->rp_fv_auw, "unknown realm",
-				    sizeof(h->rp_fv_auw));
+			/* no unsecure bsd auth permitted */
+			if (config.bsdauth == 1 && !c[sfd].ssl) {
+				/* 403 */
+				h->x_idx = 0;
+				strlcpy(status, http_s_403, sizeof(status));
+				strlcpy(h->rp_fv_cte, http_fv_cte,
+				    sizeof(h->rp_fv_cte));
+				h->rp_status = 403;
+			} else {
+				/* get access realm */
+				if (fparse(h->rp_fv_auw, "realm", acc_file,
+				    sizeof(h->rp_fv_auw)) == -1) {
+					/* realm could not be parsed */
+					strlcpy(h->rp_fv_auw, "unknown realm",
+					    sizeof(h->rp_fv_auw));
+				}
+				/* 401 */
+				strlcpy(status, http_s_401, sizeof(status));
+				strlcpy(h->rp_fv_cte, http_fv_cte,
+				    sizeof(h->rp_fv_cte)); 
+				h->x_cgi = 0;
+				h->rp_status = 401;
 			}
-			/* 401 */
-			strlcpy(status, http_s_401, sizeof(status));
-			strlcpy(h->rp_fv_cte, http_fv_cte,
-			    sizeof(h->rp_fv_cte)); 
-			h->x_cgi = 0;
-			h->rp_status = 401;
 		}
 	}
 
@@ -1624,13 +1680,13 @@ http_header(const char *header_data, const char *force_status,
 	/* Last-Modified:, Content-Length: */
 	if ((h->rp_status == 200 || h->rp_status == 206) && h->x_cgi == 0 &&
 	    h->x_idx == 0) {
-		snprintf(tmp, sizeof(tmp), "%s %s\r\n%s %d\r\n",
+		snprintf(tmp, sizeof(tmp), "%s %s\r\n%s %lld\r\n",
 		    http_fn_lmd, h->rp_fv_dam, http_fn_clt, h->rp_fsize);
 		strlcat(h->rp_header, tmp, sizeof(h->rp_header));
 	}
 	/* Content-Range: */
 	if (h->rp_status == 206 && h->x_cgi == 0 && h->x_idx == 0) {
-		snprintf(tmp, sizeof(tmp), "%s bytes %d-%d/%d\r\n",
+		snprintf(tmp, sizeof(tmp), "%s bytes %lld-%lld/%lld\r\n",
 		    http_fn_rac, h->rp_foffs, fsize - 1, fsize);
 		strlcat(h->rp_header, tmp, sizeof(h->rp_header));
 	}
