@@ -27,6 +27,7 @@
 #include <err.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <grp.h>
 #include <netdb.h>
 #include <openssl/ssl.h>
 #include <pwd.h>
@@ -59,6 +60,7 @@ static const char	*configfile = "/var/nostromo/conf/nhttpd.conf";
 volatile sig_atomic_t	quit = 0;
 int			debug = 0;
 int			mimes_size;
+int			fdbsdauth[2];
 char			*mimes;
 char			logn[1024];
 
@@ -102,7 +104,7 @@ const char *http_fn_auw		= "WWW-Authenticate:";
 const char *http_fn_ran		= "Range:";
 const char *http_fn_rac		= "Content-Range:";
 const char *http_fn_aen		= "Accept-Encoding:";
-const char *http_fv_srv		= "nostromo 1.7.9";
+const char *http_fv_srv		= "nostromo 1.8.2";
 const char *http_fv_pro		= "HTTP/1.1";
 const char *http_fv_cgi		= "CGI/1.1";
 const char *http_fv_teg		= "chunked";
@@ -386,12 +388,13 @@ main(int argc, char *argv[])
 	int			sdlis1, sdlis2, sdlis3, sdlis4;
 	int			sdmax, sdnew, sdnow, sdlim;
 	int			i, j, k, r, s, ch, rd, rt, wr, post, size;
-	int			fd, erro, opt_c, opt_r, opt_4, opt_6;
+	int			fd, erro, pid, opt_c, opt_r, opt_4, opt_6;
 	char			in[HDN * HDS + 1], tmp[HDS + 1];
 	char			header[HDN][HDS + 1];
 	char			*b, *h, *offset, *body = NULL;
 	extern char		**environ;
 	struct passwd		*pwd = NULL;
+	struct group		*grp = NULL;
 	struct stat		st;
 	struct hostent		*host;
 	struct sockaddr_in	sa, ca;
@@ -528,6 +531,18 @@ main(int argc, char *argv[])
 	if (fparse(config.htpasswd, "htpasswd", config.file,
 	    sizeof(config.htpasswd)) == -1)
 		strlcpy(config.htpasswd, "0", sizeof(config.htpasswd));
+#ifdef __OpenBSD__
+	else {
+		if (strcmp(config.htpasswd, "+bsdauth") == 0)
+			config.bsdauth = 1;
+		else if (strcmp(config.htpasswd, "+bsdauthnossl") == 0)
+			config.bsdauth = 2;
+		else
+			config.bsdauth = 0;
+	}
+#else
+	config.bsdauth = 0;
+#endif
 	if (fparse(config.sslportc, "sslport", config.file,
 	    sizeof(config.sslportc)) == -1)
 		config.ssl = 0;
@@ -555,6 +570,9 @@ main(int argc, char *argv[])
 	if (fparse(config.c404, "custom_404", config.file, sizeof(config.user))
 	    == -1)
 		strlcpy(config.c404, "0", sizeof(config.c404));
+	if (fparse(config.homedirs, "homedirs", config.file,
+	    sizeof(config.homedirs)) == -1)
+		strlcpy(config.homedirs, "0", sizeof(config.homedirs));
 
 	/* we need all mandatory parameters otherwise no start */
 	if (r == -1)
@@ -868,6 +886,28 @@ main(int argc, char *argv[])
 	dup(0);
 	dup(0);
 
+        /*
+         * basic authentication via bsd authentication framework
+         */
+        if (config.bsdauth) {
+		pipe(fdbsdauth);
+		pid = fork();
+		if (pid == -1)
+			err(1, "fork");
+		/* bsd auth daemon */
+		if (pid == 0) {
+			/* child closes all fds except pipe to parent */
+			sys_close_except(fdbsdauth[1]);
+			grp = getgrnam("auth");
+			setgid(grp->gr_gid);
+			if (config.user[0] != '0')
+				setuid(pwd->pw_uid);
+			sys_daemon_bsdauth();
+			exit(0);
+		}
+		close(fdbsdauth[1]);
+	}
+
 	/*
 	 * chroot to serverroot directory
 	 */
