@@ -29,6 +29,10 @@
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
+#ifdef __OpenBSD__
+#include <bsd_auth.h>
+#include <login_cap.h>
+#endif
 
 #include "../libmy/str.h"
 #include "../libmy/file.h"
@@ -145,6 +149,68 @@ sys_access_auth(const char *clientuser, const char *clientpw)
 	return (r);
 }
 
+/*
+ * sys_access_bsdauth()
+ *	basic authentication over the bsd authentication framework
+ * Return:
+ *	0 = authentication failed, 1 = authentication successfull
+ */
+int
+sys_access_bsdauth(const char *clientuser, const char *clientpw)
+{
+	int	r, clientuserlen, clientpwlen;
+	char	answer[4];
+
+	r = 0;
+
+	clientuserlen = strlen(clientuser);
+	clientpwlen = strlen(clientpw);
+	if (clientuserlen == 0 || clientpwlen == 0)
+		return (r);
+
+	sys_write(fdbsdauth[0], clientuser, clientuserlen);
+	sys_write(fdbsdauth[0], clientpw, clientpwlen);
+	memset(answer, 0, sizeof(answer));
+	sys_read(fdbsdauth[0], answer, sizeof(answer));
+	if (strcmp(answer, "ok") == 0)
+		r = 1;
+
+	return (r);
+}
+
+/*
+ * sys_daemon_bsdauth()
+ *	this function gets forked to a own process and handles
+ *	basic authentication requests via the bsd authentication
+ *	framework
+ * Return:
+ *	none
+ */
+void
+sys_daemon_bsdauth(void)
+{
+	int	r;
+	char	clientuser[128], clientpw[128];
+
+	r = 0;
+
+	while (!quit) {
+		memset(clientuser, 0, sizeof(clientuser));
+		memset(clientpw, 0, sizeof(clientpw));
+		if (sys_read(fdbsdauth[1], clientuser, sizeof(clientuser)) < 1)
+			continue;
+		if (sys_read(fdbsdauth[1], clientpw, sizeof(clientpw)) < 1)
+			continue;
+#ifdef __OpenBSD__
+		r = auth_userokay(clientuser, NULL, NULL, clientpw);
+#endif
+		if (r > 0)
+			sys_write(fdbsdauth[1], "ok", 2);
+		else
+			sys_write(fdbsdauth[1], "nok", 3);
+	}
+}
+
 /*
  * sys_read()
  *	synchronous read
@@ -500,11 +566,11 @@ sys_log(const char *file, const int debug, const char *string, ...)
 	struct tm	*t;
 
 	if (!debug)
-		return 0;
+		return (0);
 
 	if ((sfile = fopen(file, "a")) == NULL)
-		return -1;
-	
+		return (-1);
+
 	time(&tnow);
 	t = localtime(&tnow);
 	d = asctime(t);
@@ -517,7 +583,19 @@ sys_log(const char *file, const int debug, const char *string, ...)
 
 	fclose(sfile);
 
-	return 0;
+	return (0);
+}
+
+/*
+ * sys_compar()
+ *	compare function for qsort
+ * Return:
+ *	>-1 = success
+ */
+int
+sys_compar(const void *p, const void *q)
+{
+	return (strcmp(*(char **)p, *(char **)q));
 }
 
 /*
